package gov.noaa.esrl.mcidas;

import edu.wisc.ssec.mcidas.AreaFileException;
import ucar.ma2.Array;
import ucar.ma2.ArrayFloat;
import ucar.ma2.ArrayInt;
import ucar.ma2.Index;

public class AreaImageWriter extends AreaDataWriter
    {

    public AreaImageWriter(AreaFileReader afr, NetCDFHandler nh) throws AreaFileException
        {
        super(afr, nh);
        }
    
    // Write non-GVAR data (and lat/lons) for arbitrary subsector of Area image
    public void writeData()
        {
        int[] imageOrigin = new int[2];
        ArrayInt.D2 imageLine_nc = new ArrayInt.D2(numBands, numElements_nc);

        int[] llOrigin = new int[2];
        llOrigin[0] = 0;
        llOrigin[1] = 0;
        float linEles[][] = new float[2][1];
        float latLons[][] = new float[2][1];
        Array aLats = new ArrayFloat.D2(1, numElements_nc);
        Array aLons = new ArrayFloat.D2(1, numElements_nc);
        Index iLats = aLats.getIndex();
        Index iLons = aLons.getIndex();        
        
        int pixelValue;
        int shift = 0;    // Assume no bit shifting for non-GVAR data (may need to adjust...)
        int iLine_area;
        int iElem_area;
        int iStartLine_area = 0;
        int iStartElem_area = 0;
                
        if (startLine_nc != 0) 
            iStartLine_area = Math.round((startLine_nc - startLine_area) / (float)lineRes_area);
        if (startElement_nc != 0) 
            iStartElem_area = Math.round((startElement_nc - startElement_area) / (float)elemRes_area);

        if (lineRes_nc != lineRes_area) lineStep = lineRes_nc / lineRes_area;
        if (elemRes_nc != elemRes_area) elemStep = elemRes_nc / elemRes_area;
        if (lineStep == 0) lineStep = 1;
        if (elemStep == 0) elemStep = 1;
        iLine_area = iStartLine_area - lineStep;
        
        logger.info("start_line_area, numLines: " + iStartLine_area  + "  " + numLines_nc);
        logger.info("start_elem_area, numElems: " + iStartElem_area  + "  " + numElements_nc);      
        logger.info("lineStep, elemStep (rel to area): " + lineStep  + "  " + elemStep);    
        logger.info("write latlons:  " + haveLatLons);
        
        try
            {            
            // Loop through lines in netCDF file
            for (int iLine_nc = 0; iLine_nc < numLines_nc; iLine_nc++)
                {                
                iLine_area += lineStep;
                
                // Fill with 'missing' if line is outside the Area
                if (iLine_area < 0 || iLine_area >= numLines_area ) 
                    {
                    for (int iElem_nc = 0; iElem_nc < numElements_nc; iElem_nc++)
                        {
                        latLons[0][0] = Float.NaN;
                        latLons[1][0] = Float.NaN;   
                        if (haveLatLons)
                            {
                            aLats.setFloat(iLats.set(0, iElem_nc), (float) latLons[0][0]);
                            aLons.setFloat(iLons.set(0, iElem_nc), (float) latLons[1][0]);
                            }
                       
                        if (dataWidth == 1)
                            imageLine_nc.set(0, iElem_nc, (byte) 0);
                        else if (dataWidth == 2)
                            imageLine_nc.set(0, iElem_nc, (short) 0);
                        else if (dataWidth == 4)
                            imageLine_nc.set(0, iElem_nc, (int) 0);
                        }
                    }               
                else   // Line is within the Area
                    {
                    iElem_area = iStartElem_area - elemStep;
                    linEles[1][0] = iLine_area;

                    // Loop through elements in netCDF file
                    for (int iElem_nc = 0; iElem_nc < numElements_nc; iElem_nc++)
                        {
                        iElem_area += elemStep;

                        // Fill with 'missing' if element is outside the Area
                        if (iElem_area < 0 || iElem_area >= numElements_area)
                            {
                            pixelValue = 0;
                            latLons[0][0] = Float.NaN;
                            latLons[1][0] = Float.NaN;
                            }
                        else      // Element is within the Area
                            {
                            pixelValue = area_data[0][iLine_area][iElem_area];
                            if (haveLatLons)
                                {
                                linEles[0][0] = iElem_area;
                                latLons = nav.toLatLon(linEles);
                                }
                            }

                        if (haveLatLons)
                            {
                            aLats.setFloat(iLats.set(0, iElem_nc),
                                    (float) latLons[0][0]);
                            aLons.setFloat(iLons.set(0, iElem_nc),
                                    (float) latLons[1][0]);
                            }

                        if (dataWidth == 1)
                            imageLine_nc.set(0, iElem_nc, (byte) pixelValue);
                        else if (dataWidth == 2)
                            {
                            // Need to shift GVAR
                            pixelValue = (pixelValue >> shift);
                            imageLine_nc.set(0, iElem_nc, (short) pixelValue);
                            }
                        else if (dataWidth == 4)
                            {
                            pixelValue = (pixelValue >> shift);
                            imageLine_nc.set(0, iElem_nc, (int) pixelValue);
                            }
                    /*     
                          if (iElem_nc >= 200 && iElem_nc < 230 && iLine_nc >=
                          250 && iLine_nc < 280) 
                              {
                              System.out.println("nc: " +
                                      iLine_nc + " " + iElem_nc + " area: " + iLine_area + " " +
                                      iElem_area + " " + " swapped: " + pixelValue + " " +
                                      Integer.toHexString(pixelValue) + " Lat/lon: " +
                                      latLons[0][0] + " " + latLons[1][0]);
                              }
                    */     
                        }
                    }
                
                // Write the line's data into the netCDF file
                imageOrigin[0] = iLine_nc;
                imageOrigin[1] = 0;
                nh.writeArrayData("image", imageOrigin, imageLine_nc);

                if (haveLatLons)
                    {
                    llOrigin[0] = iLine_nc;
                    nh.writeArrayData(latitude, llOrigin, aLats);
                    nh.writeArrayData(longitude, llOrigin, aLons);
                    }
                }
            }
        catch (Exception e)
            {
            logger.error("Can't write image data: ", e); 
            }
        }
    
    public void writeNavAndCalVars()
        {
        logger.info("No Nav and Cal vars written for this image");
        }        

    }
